bom_data = {
    ("iron_bf", "Pellets_high", "Coke", "Energy", "Bio-PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_high", "Coke", "Energy", "Coke", "Reductant"): {"Unit": "t/t product", "Value": 0.4445},
    ("iron_bf", "Pellets_high", "Coke", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 28.073579404523805,
    },
    ("iron_bf", "Pellets_high", "Coke", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 1.8302102199789223,
    },
    ("iron_bf", "Pellets_high", "Coke", "Energy", "Hydrogen", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_high", "Coke", "Energy", "PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_high", "Coke", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.019299934640706815,
    },
    ("iron_bf", "Pellets_high", "Coke", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.047284839869731705,
    },
    ("iron_bf", "Pellets_high", "Coke", "Materials", "Pellets_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.3875335131550575,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI", "Energy", "Bio-PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_high", "Coke+PCI", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.2625832054586857,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 28.073579404523805,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 1.8302102199789223,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI", "Energy", "Hydrogen", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_high", "Coke+PCI", "Energy", "PCI", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.20212977171257143,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.019299934640706815,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.047284839869731705,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI", "Materials", "Pellets_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.3875335131550575,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Energy", "Bio-PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.2625832054586857,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 28.073579404523805,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 1.8302102199789223,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Energy", "Hydrogen", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.02573411445414683,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Energy", "PCI", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.08983545409447617,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.019299934640706815,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.047284839869731705,
    },
    ("iron_bf", "Pellets_high", "Coke+PCI+H2", "Materials", "Pellets_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.3875335131550575,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Energy", "Bio-PCI", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.20212977171257143,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.2625832054586857,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 28.073579404523805,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 1.8302102199789223,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Energy", "Hydrogen", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.0,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Energy", "PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.019299934640706815,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.047284839869731705,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI", "Materials", "Pellets_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.3875335131550575,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Energy", "Bio-PCI", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.08983545409447617,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.2625832054586857,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 28.073579404523805,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 1.8302102199789223,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Energy", "Hydrogen", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.02573411445414683,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Energy", "PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.019299934640706815,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.047284839869731705,
    },
    ("iron_bf", "Pellets_high", "Coke+bio-PCI+H2", "Materials", "Pellets_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.3875335131550575,
    },
    ("iron_bf", "Pellets_high", "null", "Constraint", "Pellets_high", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_bf", "Pellets_high", "null", "Constraint", "Pellets_high", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke", "Energy", "Bio-PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.49999999999999994,
    },
    ("iron_bf", "Pellets_mid", "Coke", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 29.999999999999996,
    },
    ("iron_bf", "Pellets_mid", "Coke", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t product", "Value": 2.2},
    ("iron_bf", "Pellets_mid", "Coke", "Energy", "Hydrogen", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke", "Energy", "PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.0602373338317374,
    },
    ("iron_bf", "Pellets_mid", "Coke", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.14758146788775664,
    },
    ("iron_bf", "Pellets_mid", "Coke", "Materials", "Pellets_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.482746635006723,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Energy", "Bio-PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.3055999999999999,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 29.999999999999996,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t product", "Value": 2.2},
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Energy", "Hydrogen", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Energy", "PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.216},
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.0602373338317374,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.14758146788775664,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI", "Materials", "Pellets_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.482746635006723,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Energy", "Bio-PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.3055999999999999,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 29.999999999999996,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t product", "Value": 2.2},
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Energy", "Hydrogen", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.027500000000000004,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Energy", "PCI", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.09599999999999999,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.0602373338317374,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.14758146788775664,
    },
    ("iron_bf", "Pellets_mid", "Coke+PCI+H2", "Materials", "Pellets_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.482746635006723,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Energy", "Bio-PCI", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.216,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.3055999999999999,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 29.999999999999996,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t product", "Value": 2.2},
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Energy", "Hydrogen", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.0,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Energy", "PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.0602373338317374,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.14758146788775664,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI", "Materials", "Pellets_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.482746635006723,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Energy", "Bio-PCI", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.09599999999999999,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Energy", "Coke", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.3055999999999999,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 29.999999999999996,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t product", "Value": 2.2},
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Energy", "Hydrogen", "Reductant"): {
        "Unit": "t/t product",
        "Value": 0.027500000000000004,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Energy", "PCI", "Reductant"): {"Unit": "t/t product", "Value": 0.0},
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.0602373338317374,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.14758146788775664,
    },
    ("iron_bf", "Pellets_mid", "Coke+bio-PCI+H2", "Materials", "Pellets_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.482746635006723,
    },
    ("iron_bf", "Pellets_mid", "null", "Constraint", "Pellets_mid", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_bf", "Pellets_mid", "null", "Constraint", "Pellets_mid", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_bf", "null", "Coke", "Constraint", "Coke", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_bf", "null", "Coke", "Constraint", "Coke", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_bf", "null", "Coke+PCI", "Constraint", "Coke+PCI", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_bf", "null", "Coke+PCI", "Constraint", "Coke+PCI", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_bf", "null", "Coke+PCI+H2", "Constraint", "Coke+PCI+H2", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_bf", "null", "Coke+PCI+H2", "Constraint", "Coke+PCI+H2", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_bf", "null", "Coke+bio-PCI", "Constraint", "Coke+bio-PCI", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_bf", "null", "Coke+bio-PCI", "Constraint", "Coke+bio-PCI", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_bf", "null", "Coke+bio-PCI+H2", "Constraint", "Coke+bio-PCI+H2", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_bf", "null", "Coke+bio-PCI+H2", "Constraint", "Coke+bio-PCI+H2", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_dri", "Pellets_high", "Coal", "Energy", "Coal", "Heat"): {"Unit": "GJ/t product", "Value": 0.0},
    ("iron_dri", "Pellets_high", "Coal", "Energy", "Coal", "Reductant"): {"Unit": "GJ/t product", "Value": 16.3},
    ("iron_dri", "Pellets_high", "Coal", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 298.4,
    },
    ("iron_dri", "Pellets_high", "Coal", "Materials", "Pellets_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.328714157706093,
    },
    ("iron_dri", "Pellets_high", "Hydrogen", "Energy", "Electricity", "Heat"): {
        "Unit": "kWh/t product",
        "Value": 384.55693195398425,
    },
    ("iron_dri", "Pellets_high", "Hydrogen", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 120.0,
    },
    ("iron_dri", "Pellets_high", "Hydrogen", "Energy", "Hydrogen", "Reductant"): {
        "Unit": "GJ/t product",
        "Value": 5.666592,
    },
    ("iron_dri", "Pellets_high", "Hydrogen", "Energy", "Natural gas", "Other"): {
        "Unit": "GJ/t product",
        "Value": 0.8821620295268815,
    },
    ("iron_dri", "Pellets_high", "Hydrogen", "Materials", "Pellets_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.328714157706093,
    },
    ("iron_dri", "Pellets_high", "Natural gas", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 120.0,
    },
    ("iron_dri", "Pellets_high", "Natural gas", "Energy", "Natural gas", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 0.0,
    },
    ("iron_dri", "Pellets_high", "Natural gas", "Energy", "Natural gas", "Reductant"): {
        "Unit": "GJ/t product",
        "Value": 11.0,
    },
    ("iron_dri", "Pellets_high", "Natural gas", "Materials", "Pellets_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.328714157706093,
    },
    ("iron_dri", "Pellets_high", "null", "Constraint", "Pellets_high", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_dri", "Pellets_high", "null", "Constraint", "Pellets_high", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_dri", "Pellets_mid", "Coal", "Energy", "Coal", "Heat"): {"Unit": "GJ/t product", "Value": 0.0},
    ("iron_dri", "Pellets_mid", "Coal", "Energy", "Coal", "Reductant"): {"Unit": "GJ/t product", "Value": 16.3},
    ("iron_dri", "Pellets_mid", "Coal", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 298.4,
    },
    ("iron_dri", "Pellets_mid", "Coal", "Materials", "Pellets_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.2845098566308244,
    },
    ("iron_dri", "Pellets_mid", "Hydrogen", "Energy", "Electricity", "Heat"): {
        "Unit": "kWh/t product",
        "Value": 347.8908907688144,
    },
    ("iron_dri", "Pellets_mid", "Hydrogen", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 120.0,
    },
    ("iron_dri", "Pellets_mid", "Hydrogen", "Energy", "Hydrogen", "Reductant"): {
        "Unit": "GJ/t product",
        "Value": 5.126304000000001,
    },
    ("iron_dri", "Pellets_mid", "Hydrogen", "Energy", "Natural gas", "Other"): {
        "Unit": "GJ/t product",
        "Value": 0.7715012824817205,
    },
    ("iron_dri", "Pellets_mid", "Hydrogen", "Materials", "Pellets_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.2845098566308244,
    },
    ("iron_dri", "Pellets_mid", "Natural gas", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 120.0,
    },
    ("iron_dri", "Pellets_mid", "Natural gas", "Energy", "Natural gas", "Heat"): {"Unit": "GJ/t product", "Value": 0.0},
    ("iron_dri", "Pellets_mid", "Natural gas", "Energy", "Natural gas", "Reductant"): {
        "Unit": "GJ/t product",
        "Value": 11.000000000000002,
    },
    ("iron_dri", "Pellets_mid", "Natural gas", "Materials", "Pellets_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.2845098566308244,
    },
    ("iron_dri", "Pellets_mid", "null", "Constraint", "Pellets_mid", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_dri", "Pellets_mid", "null", "Constraint", "Pellets_mid", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_dri", "null", "Coal", "Constraint", "Coal", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_dri", "null", "Coal", "Constraint", "Coal", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_dri", "null", "Hydrogen", "Constraint", "Hydrogen", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_dri", "null", "Hydrogen", "Constraint", "Hydrogen", "Minium"): {"Unit": "%", "Value": 0.0},
    ("iron_dri", "null", "Natural gas", "Constraint", "Natural gas", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("iron_dri", "null", "Natural gas", "Constraint", "Natural gas", "Minium"): {"Unit": "%", "Value": 0.0},
    ("prep_coke", "null", "Coking coal", "Constraint", "Coking coal", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("prep_coke", "null", "Coking coal", "Constraint", "Coking coal", "Minium"): {"Unit": "%", "Value": 1.0},
    ("prep_coke", "null", "Coking coal", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t coke",
        "Value": 5.555555555555555,
    },
    ("prep_coke", "null", "Coking coal", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t coke",
        "Value": 3.9800000000000004,
    },
    ("prep_coke", "null", "Coking coal", "Materials", "Coking coal", "Feedstock"): {"Unit": "t/t coke", "Value": 1.22},
    ("prep_pellet", "IO_high", "null", "Constraint", "IO_high", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("prep_pellet", "IO_high", "null", "Constraint", "IO_high", "Minium"): {"Unit": "%", "Value": 0.0},
    ("prep_pellet", "IO_high", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t pellet",
        "Value": 41.0,
    },
    ("prep_pellet", "IO_high", "null", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t pellet", "Value": 0.42},
    ("prep_pellet", "IO_high", "null", "Materials", "IO_high", "Feedstock"): {"Unit": "t/t pellet", "Value": 1.0},
    ("prep_pellet", "IO_mid", "null", "Constraint", "IO_mid", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("prep_pellet", "IO_mid", "null", "Constraint", "IO_mid", "Minium"): {"Unit": "%", "Value": 0.0},
    ("prep_pellet", "IO_mid", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t pellet",
        "Value": 41.0,
    },
    ("prep_pellet", "IO_mid", "null", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t pellet", "Value": 1.06},
    ("prep_pellet", "IO_mid", "null", "Materials", "IO_mid", "Feedstock"): {"Unit": "t/t pellet", "Value": 1.0},
    ("steel_bof", "DRI_high", "null", "Constraint", "DRI_high", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_bof", "DRI_high", "null", "Constraint", "DRI_high", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_bof", "DRI_high", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 37.44360073652292,
    },
    ("steel_bof", "DRI_high", "null", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 1.2107134448283747,
    },
    ("steel_bof", "DRI_high", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.03675880284439286,
    },
    ("steel_bof", "DRI_high", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.10660052824873928,
    },
    ("steel_bof", "DRI_high", "null", "Materials", "DRI_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.177116256203829,
    },
    ("steel_bof", "DRI_mid", "null", "Constraint", "DRI_mid", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_bof", "DRI_mid", "null", "Constraint", "DRI_mid", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_bof", "DRI_mid", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 44.610291894000206,
    },
    ("steel_bof", "DRI_mid", "null", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 1.4424435447283803,
    },
    ("steel_bof", "DRI_mid", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.12365484407093384,
    },
    ("steel_bof", "DRI_mid", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.35859904780570817,
    },
    ("steel_bof", "DRI_mid", "null", "Materials", "DRI_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.4024158667840196,
    },
    ("steel_bof", "HBI_high", "null", "Constraint", "HBI_high", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_bof", "HBI_high", "null", "Constraint", "HBI_high", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_bof", "HBI_high", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 37.44360073652292,
    },
    ("steel_bof", "HBI_high", "null", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 1.7112671408561018,
    },
    ("steel_bof", "HBI_high", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.03675880284439286,
    },
    ("steel_bof", "HBI_high", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.10660052824873928,
    },
    ("steel_bof", "HBI_high", "null", "Materials", "HBI_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.177116256203829,
    },
    ("steel_bof", "HBI_mid", "null", "Constraint", "HBI_mid", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_bof", "HBI_mid", "null", "Constraint", "HBI_mid", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_bof", "HBI_mid", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 44.610291894000206,
    },
    ("steel_bof", "HBI_mid", "null", "Energy", "Flexible", "Heat"): {
        "Unit": "GJ/t product",
        "Value": 2.0388030307068954,
    },
    ("steel_bof", "HBI_mid", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.12365484407093384,
    },
    ("steel_bof", "HBI_mid", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.35859904780570817,
    },
    ("steel_bof", "HBI_mid", "null", "Materials", "HBI_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.4024158667840196,
    },
    ("steel_bof", "Hot metal", "null", "Constraint", "Hot metal", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_bof", "Hot metal", "null", "Constraint", "Hot metal", "Minium"): {"Unit": "%", "Value": 0.7},
    ("steel_bof", "Hot metal", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 34.85817316437359,
    },
    ("steel_bof", "Hot metal", "null", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t product", "Value": 0.0},
    ("steel_bof", "Hot metal", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.008615554298366596,
    },
    ("steel_bof", "Hot metal", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.024985107465263126,
    },
    ("steel_bof", "Hot metal", "null", "Materials", "Hot metal", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.095838046722067,
    },
    ("steel_bof", "Pig iron", "null", "Constraint", "Pig iron", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_bof", "Pig iron", "null", "Constraint", "Pig iron", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_bof", "Pig iron", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 34.85817316437359,
    },
    ("steel_bof", "Pig iron", "null", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t product", "Value": 0.0},
    ("steel_bof", "Pig iron", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.008615554298366596,
    },
    ("steel_bof", "Pig iron", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.024985107465263126,
    },
    ("steel_bof", "Pig iron", "null", "Materials", "Pig iron", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.095838046722067,
    },
    ("steel_bof", "Scrap", "null", "Constraint", "Scrap", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_bof", "Scrap", "null", "Constraint", "Scrap", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_bof", "Scrap", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 32.62745690056601,
    },
    ("steel_bof", "Scrap", "null", "Energy", "Flexible", "Heat"): {"Unit": "GJ/t product", "Value": 1.4911572013739554},
    ("steel_bof", "Scrap", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.004838525964379703,
    },
    ("steel_bof", "Scrap", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.014031725296701138,
    },
    ("steel_bof", "Scrap", "null", "Materials", "Scrap", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.0257109135015452,
    },
    ("steel_eaf", "DRI_high", "null", "Constraint", "DRI_high", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_eaf", "DRI_high", "null", "Constraint", "DRI_high", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_eaf", "DRI_high", "null", "Energy", "Electricity", "Heat"): {
        "Unit": "kWh/t product",
        "Value": 326.4081480016136,
    },
    ("steel_eaf", "DRI_high", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 37.44360073652292,
    },
    ("steel_eaf", "DRI_high", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.03675880284439286,
    },
    ("steel_eaf", "DRI_high", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.10660052824873928,
    },
    ("steel_eaf", "DRI_high", "null", "Materials", "DRI_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.177116256203829,
    },
    ("steel_eaf", "DRI_mid", "null", "Constraint", "DRI_mid", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_eaf", "DRI_mid", "null", "Constraint", "DRI_mid", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_eaf", "DRI_mid", "null", "Energy", "Electricity", "Heat"): {
        "Unit": "kWh/t product",
        "Value": 411.9179525314357,
    },
    ("steel_eaf", "DRI_mid", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 44.610291894000206,
    },
    ("steel_eaf", "DRI_mid", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.12365484407093384,
    },
    ("steel_eaf", "DRI_mid", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.35859904780570817,
    },
    ("steel_eaf", "DRI_mid", "null", "Materials", "DRI_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.4024158667840196,
    },
    ("steel_eaf", "HBI_high", "null", "Constraint", "HBI_high", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_eaf", "HBI_high", "null", "Constraint", "HBI_high", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_eaf", "HBI_high", "null", "Energy", "Electricity", "Heat"): {
        "Unit": "kWh/t product",
        "Value": 420.4856843930561,
    },
    ("steel_eaf", "HBI_high", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 37.44360073652292,
    },
    ("steel_eaf", "HBI_high", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.03675880284439286,
    },
    ("steel_eaf", "HBI_high", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.10660052824873928,
    },
    ("steel_eaf", "HBI_high", "null", "Materials", "HBI_high", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.177116256203829,
    },
    ("steel_eaf", "HBI_mid", "null", "Constraint", "HBI_mid", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_eaf", "HBI_mid", "null", "Constraint", "HBI_mid", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_eaf", "HBI_mid", "null", "Energy", "Electricity", "Heat"): {
        "Unit": "kWh/t product",
        "Value": 525.8011511724798,
    },
    ("steel_eaf", "HBI_mid", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 44.610291894000206,
    },
    ("steel_eaf", "HBI_mid", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.12365484407093384,
    },
    ("steel_eaf", "HBI_mid", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.35859904780570817,
    },
    ("steel_eaf", "HBI_mid", "null", "Materials", "HBI_mid", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.4024158667840196,
    },
    ("steel_eaf", "Hot metal", "null", "Constraint", "Hot metal", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_eaf", "Hot metal", "null", "Constraint", "Hot metal", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_eaf", "Hot metal", "null", "Energy", "Electricity", "Heat"): {"Unit": "kWh/t product", "Value": 0.0},
    ("steel_eaf", "Hot metal", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 34.85817316437359,
    },
    ("steel_eaf", "Hot metal", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.008615554298366596,
    },
    ("steel_eaf", "Hot metal", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.024985107465263126,
    },
    ("steel_eaf", "Hot metal", "null", "Materials", "Hot metal", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.095838046722067,
    },
    ("steel_eaf", "Pig iron", "null", "Constraint", "Pig iron", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_eaf", "Pig iron", "null", "Constraint", "Pig iron", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_eaf", "Pig iron", "null", "Energy", "Electricity", "Heat"): {"Unit": "kWh/t product", "Value": 0.0},
    ("steel_eaf", "Pig iron", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 34.85817316437359,
    },
    ("steel_eaf", "Pig iron", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.008615554298366596,
    },
    ("steel_eaf", "Pig iron", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.024985107465263126,
    },
    ("steel_eaf", "Pig iron", "null", "Materials", "Pig iron", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.095838046722067,
    },
    ("steel_eaf", "Scrap", "null", "Constraint", "Scrap", "Maximum"): {"Unit": "%", "Value": 1.0},
    ("steel_eaf", "Scrap", "null", "Constraint", "Scrap", "Minium"): {"Unit": "%", "Value": 0.0},
    ("steel_eaf", "Scrap", "null", "Energy", "Electricity", "Heat"): {
        "Unit": "kWh/t product",
        "Value": 339.68561047298704,
    },
    ("steel_eaf", "Scrap", "null", "Energy", "Electricity", "Machine drive"): {
        "Unit": "kWh/t product",
        "Value": 32.62745690056601,
    },
    ("steel_eaf", "Scrap", "null", "Materials", "Burnt dolomite", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.004838525964379703,
    },
    ("steel_eaf", "Scrap", "null", "Materials", "Burnt lime", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 0.014031725296701138,
    },
    ("steel_eaf", "Scrap", "null", "Materials", "Scrap", "Feedstock"): {
        "Unit": "t/t product",
        "Value": 1.0257109135015452,
    },
}
